import java.awt.*; 
import java.io.IOException; 
import java.net.*; 
import java.util.Vector; 
 
import javax.media.*; 
import javax.media.format.*;
import javax.media.control.QualityControl; 
import javax.media.control.TrackControl; 
import javax.media.protocol.ContentDescriptor; 
import javax.media.protocol.DataSource; 
import javax.media.rtp.*; 
import javax.media.rtp.event.*; 
import javax.swing.JFrame; 
import javax.swing.JPanel; 
 
public class ChatManager implements ReceiveStreamListener, SessionListener, 
        ControllerListener { 
	
	MediaLocator audioCapDevLoc = null; 
    MediaLocator videoCapDevLoc = null; 
    CaptureDeviceInfo audioCapDevInfo = null; 
    CaptureDeviceInfo videoCapDevInfo = null; 
    Vector audioCapDevList = null; 
    Vector videoCapDevList = null;
 
    String destAddress; 
    String username; 
    int destCtrlPort; 
    int destAudioDataPort; 
    int destAudioCtrlPort; 
    int destVideoDataPort; 
    int destVideoCtrlPort; 
    int ttl; 
    boolean isMulticastAddress = false; 
    
    Processor audioProcessor = null; 
    Processor videoProcessor = null; 
    DataSource audioDataOutput = null; 
    DataSource videoDataOutput = null; 
    RTPManager audioRTPManager; 
    RTPManager videoRTPManager; 
    TrackControl audioTracks[] = null; 
    TrackControl videoTracks[] = null; 
    boolean initAudioProcessorOK = false; 
    boolean initVideoProcessorOK = false; 
 
    DatagramSocket chatManagerSocket; 
    DatagramSocket audioDataSocket; 
    DatagramSocket audioCtrlSocket; 
    DatagramSocket videoDataSocket; 
    DatagramSocket videoCtrlSocket; 
 
    MainFrame mainFrame; 
    Vector playerWindows = null; 
 
    Object dataSynchronization = new Object(); 
 
    public ChatManager(String name, MainFrame chatFrame) throws SocketException { 
    	
    	audioCapDevList = CaptureDeviceManager.getDeviceList(new AudioFormat( 
                AudioFormat.LINEAR)); 
        videoCapDevList = CaptureDeviceManager.getDeviceList(new VideoFormat( 
                VideoFormat.YUV)); 
        if ((audioCapDevList.size() > 0)) { 
            audioCapDevInfo = (CaptureDeviceInfo) audioCapDevList.elementAt(0); 
            audioCapDevLoc = audioCapDevInfo.getLocator(); 
        } 
        if ((videoCapDevList.size() > 0)) { 
            videoCapDevInfo = (CaptureDeviceInfo) videoCapDevList.elementAt(0); 
            videoCapDevLoc = videoCapDevInfo.getLocator(); 
        } 
 
        this.username = name; 
        this.mainFrame = chatFrame; 
 
        playerWindows = new Vector(); 
        
        chatManagerSocket = new DatagramSocket(5000); 
        audioDataSocket = new DatagramSocket(); 
        audioCtrlSocket = new DatagramSocket(); 
        videoDataSocket = new DatagramSocket(); 
        videoCtrlSocket = new DatagramSocket(); 
        
        String msg = username + ":" + getLocalAddress() + ":" + 
                chatManagerSocket.getLocalPort() + ":" + 
                audioDataSocket.getLocalPort() + ":" + 
                audioCtrlSocket.getLocalPort() + ":" + 
                videoDataSocket.getLocalPort() + ":" + 
                videoCtrlSocket.getLocalPort(); 
        new UDPSend(chatManagerSocket, "0 " + msg, "255.255.255.255", 
                   chatManagerSocket.getLocalPort(), false).start(); 
        new UDPReceive(chatManagerSocket, mainFrame, msg).start();
    } 
 
    public void setDestAddressAndPort(String ipAddress, int destCtrlPort, 
                                      int destAudioDataPort, 
                                      int destAudioCtrlPort, 
                                      int destVideoDataPort, 
                                      int destVideoCtrlPort) { 
        this.destAddress = ipAddress; 
        this.destCtrlPort = destCtrlPort; 
        this.destAudioDataPort = destAudioDataPort; 
        this.destAudioCtrlPort = destAudioCtrlPort; 
        this.destVideoDataPort = destVideoDataPort; 
        this.destVideoCtrlPort = destVideoCtrlPort; 
    } 
 
    public void setMulticastAndTTL(int ttl) { 
        this.isMulticastAddress = true; 
        this.ttl = ttl; 
    } 
 
    public void needConnect(String msg) { 
        new UDPSend(chatManagerSocket, "2 " + msg, destAddress, destCtrlPort, false). 
                start(); 
    } 
 
    public void bye() { 
        new UDPSend(chatManagerSocket, "4 " + username + ":" + getLocalAddress(), 
                            "255.255.255.255", 5000, false).start(); 
    } 
 
    public synchronized String startAudioTransmit() { 
        System.err.println("  - Open audio RTP session for: addr: " + destAddress + 
                           " localPort: " + audioDataSocket.getLocalPort() + 
                           " destPort: " + destAudioDataPort); 
 
        String result; 
        result = createAudioDataOutput(); 
        if (result != null) { 
            System.out.println(result); 
        } 
 
        result = createAudioRTPManagers(); 
        if (result != null) { 
            return result; 
        } 
        if (audioProcessor != null) { 
 
            audioProcessor.start(); 
        } 
        return null; 
    } 
 
    public synchronized String startVideoTransmit() { 
        System.err.println("  - Open video RTP session for: addr: " + destAddress + 
                           " localPort: " + videoDataSocket.getLocalPort() + 
                           " destPort: " + destVideoDataPort); 
 
        String result; 
        result = createVideoDataOutput(); 
        if (result != null) { 
            System.out.println(result); 
        } 
 
        result = createVideoRTPManagers(); 
        if (result != null) { 
            return result; 
        } 
        if (videoProcessor != null) { 
 
            videoProcessor.start(); 
        } 
        return null; 
    } 
 
    public void closeTransmit() { 
        synchronized (this) { 
            if (audioProcessor != null) { 
                audioProcessor.stop(); 
                audioRTPManager.removeSessionListener(this); 
                audioRTPManager.removeReceiveStreamListener(this); 
                audioRTPManager.removeTargets("Session ended."); 
            } 
            if (videoProcessor != null) { 
                videoProcessor.stop(); 
                if (videoRTPManager != null) { 
                    videoRTPManager.removeSessionListener(this); 
                    videoRTPManager.removeReceiveStreamListener(this); 
                    videoRTPManager.removeTargets("Session ended."); 
                } 
            } 
            for (int i = 0; i < playerWindows.size(); i++) { 
                try { 
                    ((PlayerWindow) playerWindows.elementAt(i)).close(); 
                } catch (Exception e) {} 
            } 
            playerWindows.removeAllElements(); 
        } 
    } 
 
    public void closeVideoTransmit() { 
        synchronized (this) { 
            if (videoProcessor != null) { 
                videoProcessor.stop(); 
                videoRTPManager.removeSessionListener(this); 
                videoRTPManager.removeReceiveStreamListener(this); 
                videoRTPManager.removeTargets("Session ended."); 
            } 
        } 
    } 
 
    public String initAudioProcessor() { 
        if (audioCapDevLoc == null) { 
            return "Audio capture device locator is null"; 
        } 
 
        DataSource ds; 
        try { 
            ds = Manager.createDataSource(audioCapDevLoc); 
        } catch (Exception e) { 
            return "Couldn't create DataSource"; 
        } 
 
        try { 
            audioProcessor = Manager.createProcessor(ds); 
        } catch (NoProcessorException npe) { 
            return "Couldn't create processor"; 
        } catch (IOException ioe) { 
            return "IOException creating processor"; 
        } 
 
        boolean result = waitForState(audioProcessor, Processor.Configured); 
        if (result == false) { 
            return "Couldn't configure processor"; 
        } 
 
        audioProcessor.setContentDescriptor(new ContentDescriptor( 
                ContentDescriptor.RAW_RTP)); 
 
        audioTracks = audioProcessor.getTrackControls(); 
        if (audioTracks == null || audioTracks.length < 1) { 
            return "Couldn't find tracks in processor"; 
        } 
 
        Format supported[]; 
 
        if (audioTracks[0].isEnabled()) { 
 
            supported = audioTracks[0].getSupportedFormats(); 
            
            if (supported.length > 0) { 
                audioTracks[0].setFormat(supported[0]);  
            } else { 
                audioTracks[0].setEnabled(false); 
            } 
 
        } else { 
            audioTracks[0].setEnabled(false); 
        } 
 
        initAudioProcessorOK = true; 
 
        return null; 
    } 
 
    public String initVideoProcessor() { 
 
        if (videoCapDevLoc == null) { 
            return "Video captue device locator is null"; 
        } 
 
        DataSource ds; 
        try { 
            ds = Manager.createDataSource(videoCapDevLoc); 
        } catch (Exception e) { 
            return "Couldn't create DataSource"; 
        } 
 
        try { 
            videoProcessor = Manager.createProcessor(ds); 
        } catch (NoProcessorException npe) { 
            return "Couldn't create processor"; 
        } catch (IOException ioe) { 
            return "IOException creating processor"; 
        } 
 
        boolean result = waitForState(videoProcessor, Processor.Configured); 
        if (result == false) { 
            return "Couldn't configure processor"; 
        } 
 
        videoTracks = videoProcessor.getTrackControls(); 
        if (videoTracks == null || videoTracks.length < 1) { 
            return "Couldn't find tracks in processor"; 
        } 
 
        videoProcessor.setContentDescriptor(new ContentDescriptor( 
                ContentDescriptor.RAW_RTP)); 
 
        Format supported[]; 
        Format chosen; 
        boolean atLeastOneTrack = false; 
 
        if (videoTracks[0].isEnabled()) { 
 
            supported = videoTracks[0].getSupportedFormats(); 
            if (supported.length > 0) { 
                chosen = checkForVideoSizes(videoTracks[0].getFormat(), 
                                            supported[0]);  
                videoTracks[0].setFormat(chosen); 
                atLeastOneTrack = true; 
            } else { 
                videoTracks[0].setEnabled(false); 
            } 
        } else { 
            videoTracks[0].setEnabled(false); 
        } 
 
        if (!atLeastOneTrack) { 
            return "Couldn't set any of the tracks to a valid RTP format"; 
        } 
 
        initVideoProcessorOK = true; 
 
        return null; 
    } 
 
    public void setAudioFormat(int index) { 
        audioTracks[0].setFormat(audioTracks[0].getSupportedFormats()[index]); 
    } 
 
    public void setVideoFormat(int index) { 
        Format fmt = videoTracks[0].getSupportedFormats()[index]; 
        Format fmt2 = checkForVideoSizes(videoTracks[0].getFormat(), fmt); 
        videoTracks[0].setFormat(fmt2); 
    } 
 
    public String createAudioDataOutput() { 
        if (initAudioProcessorOK != true) { 
            return "Couldn't init audio processor"; 
        } 
 
        boolean result = waitForState(audioProcessor, Controller.Realized); 
        if (result == false) { 
            return "Couldn't realize processor"; 
        } 
 
        audioDataOutput = audioProcessor.getDataOutput(); 
 
        return null; 
    } 
 
    public String createVideoDataOutput() { 
        if (initVideoProcessorOK != true) { 
            return "Couldn't init video processor"; 
        } 
 
        boolean result = waitForState(videoProcessor, Controller.Realized); 
        if (result == false) { 
            return "Couldn't realize processor"; 
        } 
 
        setJPEGQuality(videoProcessor, 0.5f); 
 
        videoDataOutput = videoProcessor.getDataOutput(); 
 
        return null; 
    } 
 
    public String createAudioRTPManagers() { 
 
        InetAddress ipAddr; 
        SendStream sendStream; 
 
        try { 
            if(audioRTPManager == null){ 
                audioRTPManager = RTPManager.newInstance(); 
            } 
            audioRTPManager.addSessionListener(this); 
            audioRTPManager.addReceiveStreamListener(this); 
 
 
            ipAddr = InetAddress.getByName(destAddress); 
 
            if (isMulticastAddress == false) { 
                audioRTPManager.initialize(new RTPSocketAdapter(audioDataSocket, 
                        audioCtrlSocket, ipAddr, destAudioDataPort, destAudioCtrlPort)); 
            } else { 
                audioRTPManager.initialize(new RTPSocketAdapter(ipAddr, 
                        destAudioDataPort, ttl)); 
            } 
 
            if (audioDataOutput != null) { 
                sendStream = audioRTPManager.createSendStream(audioDataOutput, 0); 
                sendStream.start(); 
                System.err.println("  - Start audio transmit "); 
            } 
        } catch (Exception e) { 
            return e.getMessage(); 
        } 
 
        return null; 
    } 
 
    public String createVideoRTPManagers() { 
 
        InetAddress ipAddr; 
        SendStream sendStream; 
 
        try { 
            if(videoRTPManager == null){ 
                videoRTPManager = RTPManager.newInstance(); 
            } 
            videoRTPManager.addSessionListener(this); 
            videoRTPManager.addReceiveStreamListener(this); 
 
            ipAddr = InetAddress.getByName(destAddress); 
 
            if (isMulticastAddress == false) { 
                videoRTPManager.initialize(new RTPSocketAdapter(videoDataSocket, 
                        videoCtrlSocket, ipAddr, destVideoDataPort, 
                        destVideoCtrlPort)); 
            } else { 
                videoRTPManager.initialize(new RTPSocketAdapter(ipAddr, 
                        destVideoDataPort, ttl)); 
            } 
 
            if (videoDataOutput != null) { 
                sendStream = videoRTPManager.createSendStream(videoDataOutput, 0); 
                sendStream.start(); 
                System.err.println("  - Start video transmit "); 
            } 
        } catch (Exception e) { 
            return e.getMessage(); 
        } 
 
        return null; 
    } 
 
    public synchronized void update(SessionEvent evt) { 
        if (evt instanceof NewParticipantEvent) { 
            Participant p = ((NewParticipantEvent) evt).getParticipant(); 
            System.err.println("  - A new participant had just joined: " + 
                               p.getCNAME()); 
        } 
    } 
 
    public synchronized void update(ReceiveStreamEvent evt) { 
 
        RTPManager mgr = (RTPManager) evt.getSource(); 
        Participant participant = evt.getParticipant(); 
        ReceiveStream stream = evt.getReceiveStream();  
 
        if (evt instanceof RemotePayloadChangeEvent) { 
            System.err.println("  - Received an RTP PayloadChangeEvent."); 
            System.err.println("Sorry, cannot handle payload change."); 
            System.exit(0); 
        } 
 
        else if (evt instanceof NewReceiveStreamEvent) { 
            try { 
                stream = ((NewReceiveStreamEvent) evt).getReceiveStream(); 
                DataSource ds = stream.getDataSource(); 
 
                RTPControl ctl = (RTPControl) ds.getControl( 
                        "javax.media.rtp.RTPControl"); 
                if (ctl != null) { 
                    System.err.println("  - Recevied new RTP stream: " + 
                                       ctl.getFormat()); 
                } else { 
                    System.err.println("  - Recevied new RTP stream"); 
                } 
 
                if (participant == null) { 
                    System.err.println( 
                            "      The sender of this stream had yet to be identified."); 
                } else { 
                    System.err.println("      The stream comes from: " + 
                                       participant.getCNAME()); 
                } 
 
                Player p = Manager.createPlayer(ds); 
                if (p == null) { 
                    return; 
                } 
 
                p.addControllerListener(this); 
                p.realize(); 
                if (isMulticastAddress && 
                    (ctl.getFormat() instanceof VideoFormat)) { 
                    PlayerWindow pw = new PlayerWindow(p, stream); //´ 
                    playerWindows.addElement(pw); 
                } 
 
                synchronized (dataSynchronization) { 
                    dataSynchronization.notifyAll(); 
                } 
 
            } catch (Exception e) { 
                System.err.println("NewReceiveStreamEvent exception " + 
                                   e.getMessage()); 
                return; 
            } 
        } 
 
        else if (evt instanceof StreamMappedEvent) { 
            if (stream != null && stream.getDataSource() != null) { 
                DataSource ds = stream.getDataSource(); 
                RTPControl ctl = (RTPControl) ds.getControl( 
                        "javax.media.rtp.RTPControl"); 
                System.err.println("  - The previously unidentified stream "); 
                if (ctl != null) { 
                    System.err.println("      " + ctl.getFormat()); 
                } 
                System.err.println("      had now been identified as sent by: " + 
                                   participant.getCNAME()); 
            } 
        } 
 
        else if (evt instanceof ByeEvent) { 
 
            System.err.println("  - Got \"bye\" from: " + participant.getCNAME()); 
            PlayerWindow pw = find(stream); 
            if (pw != null) { 
                pw.close(); 
                playerWindows.removeElement(pw); 
            } 
        } 
    } 
 
    public synchronized void controllerUpdate(ControllerEvent ce) { 
 
        Player p = (Player) ce.getSourceController(); 
 
        if (p == null) { 
            return; 
        } 
 
        if (ce instanceof RealizeCompleteEvent) { 
            if (isMulticastAddress) { 
                PlayerWindow pw = find(p); 
                if (pw != null) { 
                    pw.initialize(); 
                    pw.setVisible(true); 
                    pw.setResizable(false); 
                } 
            } else { 
                if (p.getVisualComponent() != null) { 
                    mainFrame.videoPanel.add(new PlayerPanel(p)); 
                } 
            } 
            p.start(); 
        } 
 
        if (ce instanceof ControllerErrorEvent) { 
            p.removeControllerListener(this); 
            PlayerWindow pw = find(p); 
            if (pw != null) { 
                pw.close(); 
                playerWindows.removeElement(pw); 
            } 
            System.err.println("Receiver internal error: " + ce); 
        } 
 
    } 
    
 
    PlayerWindow find(Player p) { 
        for (int i = 0; i < playerWindows.size(); i++) { 
            PlayerWindow pw = (PlayerWindow) playerWindows.elementAt(i); 
            if (pw.player == p) { 
                return pw; 
            } 
        } 
        return null; 
    } 
 
    PlayerWindow find(ReceiveStream strm) { 
        for (int i = 0; i < playerWindows.size(); i++) { 
            PlayerWindow pw = (PlayerWindow) playerWindows.elementAt(i); 
            if (pw.stream == strm) { 
                return pw; 
            } 
        } 
        return null; 
    } 
 
    class PlayerPanel extends JPanel { 
 
        Component vc, cc; 
 
        PlayerPanel(Player p) { 
            setLayout(new BorderLayout()); 
            if ((vc = p.getVisualComponent()) != null) { 
                add("Center", vc); 
            } 
        } 
 
        public Dimension getPreferredSize() { 
            int w = 0, h = 0; 
            if (isMulticastAddress) { 
                if (vc != null) { 
                    Dimension size = vc.getPreferredSize(); 
                    w = size.width; 
                    h = size.height; 
                } 
                if (cc != null) { 
                    Dimension size = cc.getPreferredSize(); 
                    if (w == 0) { 
                        w = size.width; 
                    } 
                    h += size.height; 
                } 
                if (w < 160) { 
                    w = 160; 
                } 
                return new Dimension(w, h); 
            } else { 
                return new Dimension(227, 186); 
            } 
        } 
    } 
 
 
    class PlayerWindow extends JFrame { 
 
        Player player; 
        ReceiveStream stream; 
 
        PlayerWindow(Player p, ReceiveStream strm) { 
            player = p; 
            stream = strm; 
        } 
 
        public void initialize() { 
            add(new PlayerPanel(player)); 
        } 
 
        public void close() { 
            player.close(); 
            setVisible(false); 
            dispose(); 
        } 
 
        public void addNotify() { 
            super.addNotify(); 
            pack(); 
        } 
    } 
 
 
    Format checkForVideoSizes(Format original, Format supported) { 
 
        int width, height; 
        Dimension size = ((VideoFormat) original).getSize(); 
        Format jpegFmt = new Format(VideoFormat.JPEG_RTP); 
        Format h263Fmt = new Format(VideoFormat.H263_RTP); 
 
        if (supported.matches(jpegFmt)) { 
            width = (size.width % 8 == 0 ? size.width : 
                     (int) (size.width / 8) * 8); 
            height = (size.height % 8 == 0 ? size.height : 
                      (int) (size.height / 8) * 8); 
        } else if (supported.matches(h263Fmt)) { 
            if (size.width < 128) { 
                width = 128; 
                height = 96; 
            } else if (size.width < 176) { 
                width = 176; 
                height = 144; 
            } else { 
                width = 352; 
                height = 288; 
            } 
        } else { 
            return supported; 
        } 
 
        return (new VideoFormat(null, 
                                new Dimension(width, height), 
                                Format.NOT_SPECIFIED, 
                                null, 
                                Format.NOT_SPECIFIED)).intersects(supported); 
    } 
 
    void setJPEGQuality(Player p, float val) { 
 
        Control cs[] = p.getControls(); 
        QualityControl qc = null; 
        VideoFormat jpegFmt = new VideoFormat(VideoFormat.JPEG); 
 
        for (int i = 0; i < cs.length; i++) { 
 
            if (cs[i] instanceof QualityControl && cs[i] instanceof Owned) { 
                Object owner = ((Owned) cs[i]).getOwner(); 
 
                if (owner instanceof Codec) { 
                    Format fmts[] = ((Codec) owner).getSupportedOutputFormats(null); 
                    for (int j = 0; j < fmts.length; j++) { 
                        if (fmts[j].matches(jpegFmt)) { 
                            qc = (QualityControl) cs[i]; 
                            qc.setQuality(val); 
                            System.err.println("- Setting quality to " + 
                                               val + " on " + qc); 
                            break; 
                        } 
                    } 
                } 
                if (qc != null) { 
                    break; 
                } 
            } 
        } 
    } 
 
    public Integer stateLock = new Integer(0); 
    public boolean failed = false; 
 
    Integer getStateLock() { 
        return stateLock; 
    } 
 
    void setFailed() { 
        failed = true; 
    } 
 
    public synchronized boolean waitForState(Processor p, int state) { 
        p.addControllerListener(new StateListener()); 
        failed = false; 
 
        if (state == Processor.Configured) { 
            p.configure(); 
        } else if (state == Processor.Realized) { 
            p.realize(); 
        } 
 
        while (p.getState() < state && !failed) { 
            synchronized (getStateLock()) { 
                try { 
                    getStateLock().wait(); 
                } catch (InterruptedException ie) { 
                    return false; 
                } 
            } 
        } 
 
        if (failed) { 
            return false; 
        } else { 
            return true; 
        } 
    } 
 
 
    class StateListener implements ControllerListener { 
 
        public void controllerUpdate(ControllerEvent ce) { 
 
            if (ce instanceof ControllerClosedEvent) { 
                setFailed(); 
            } 
 
            if (ce instanceof ControllerEvent) { 
                synchronized (getStateLock()) { 
                    getStateLock().notifyAll(); 
                } 
            } 
        } 
    } 
 
 
    public String getLocalAddress() { 
        InetAddress addr = null; 
        try { 
            addr = InetAddress.getLocalHost(); 
        } catch (UnknownHostException ex) { 
        } 
        return addr.getHostAddress(); 
    } 
 
} 


