import java.awt.*; 
import java.awt.event.*; 
import java.net.*;
import javax.swing.*; 
import javax.swing.table.DefaultTableModel; 
 
public class MainFrame extends JFrame{ 
    
	ChatManager chatManager = null; 
    
    boolean onVideo = false; 
 
    String username; 
    
    JPanel contentPanel; 
    JPanel GeneralPanel = new JPanel(); 
    JPanel videoPanel = new JPanel(); 
    JScrollPane jScrollPanel = new JScrollPane(); 
        
    String[] attributes = {"Name", "IP Address", "Ctrl Port", "Audio Port", "Audio RTCP Port", "Video Port", "Video RTCP Port"}; 
    DefaultTableModel defaultTableModel = new DefaultTableModel(attributes, 0); 
    JTable userTable = new JTable(defaultTableModel);
        
    JButton connectButton = new JButton(); 
    JButton disconnectButton = new JButton(); 
    JButton videoButton = new JButton(); 
    JButton multicastButton = new JButton(); 
    
    
    public MainFrame(String name) { 
        this.username = name; 
        try { 
            mainInit(); 
            setVisible(true); 
            addWindowListener( 
            new WindowAdapter() { 
                public void windowClosing(WindowEvent e) { 
                    chatManager.bye(); 
                    System.exit(0); 
                } 
            } 
            ); 
            RTPInit(); 
        } catch (Exception exception) { 
            exception.printStackTrace(); 
        } 
    } 
 
    private void mainInit() throws Exception { 
 
        setSize(new Dimension(470, 466)); 
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize(); 
        Dimension frameSize = getSize(); 
        setLocation((screenSize.width - frameSize.width) / 2, 
                    (screenSize.height - frameSize.height) / 2); 
        setTitle("Chat!"); 
        setResizable(false); 
        contentPanel = (JPanel) getContentPane(); 
        contentPanel.setBorder(null); 
        GeneralPanel.setLayout(null); 
        GeneralPanel.setBorder(null); 
        GeneralPanel.setDebugGraphicsOptions(0); 
        videoPanel.setBackground(Color.white); 
        videoPanel.setBorder(BorderFactory.createLineBorder(Color.black)); 
        videoPanel.setBounds(new Rectangle(5, 3, 450, 236)); 
        jScrollPanel.setBorder(BorderFactory.createLineBorder(Color.black)); 
        jScrollPanel.setBounds(new Rectangle(5, 245, 450, 122)); 
        userTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF); 
                
        connectButton.setBounds(new Rectangle(22, 375, 86, 23)); 
        connectButton.setText("Connect"); 
        connectButton.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) { 
                connectButton_actionPerformed(e); 
            } 
        }); 
        disconnectButton.setBounds(new Rectangle(234, 375, 86, 23)); 
        disconnectButton.setText("Disconnect"); 
        disconnectButton.setEnabled(false); 
        disconnectButton.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) { 
                dissconnectButton_actionPerformed(e); 
            } 
        }); 
        videoButton.setBounds(new Rectangle(340, 375, 86, 23)); 
        videoButton.setText("Video"); 
        videoButton.setEnabled(false); 
        videoButton.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) { 
                videoButton_actionPerformed(e); 
            } 
        }); 
        multicastButton.setBounds(new Rectangle(128, 375, 86, 23)); 
        multicastButton.setText("Multicast"); 
        multicastButton.addActionListener(new ActionListener() { 
            public void actionPerformed(ActionEvent e) { 
                multicastButton_actionPerformed(e); 
            } 
        }); 
        
        contentPanel.add(GeneralPanel, java.awt.BorderLayout.CENTER); 
        GeneralPanel.add(videoPanel); 
        GeneralPanel.add(jScrollPanel); 
        GeneralPanel.add(multicastButton); 
        GeneralPanel.add(connectButton); 
        GeneralPanel.add(disconnectButton); 
        GeneralPanel.add(videoButton); 
        jScrollPanel.getViewport().add(userTable); 
    } 
 
    public void RTPInit() throws SocketException { 
 
        chatManager = new ChatManager(username, this); 
        chatManager.initAudioProcessor(); 
        chatManager.initVideoProcessor(); 
 
    } 
 
    public void connectButton_actionPerformed(ActionEvent e) { 
        int row = userTable.getSelectedRow(); 
        if(row != -1){ 
            String[] s = new String[7]; 
 
            for (int i = 0; i < defaultTableModel.getColumnCount(); i++) 
                s[i] = (String) defaultTableModel.getValueAt(row, i); 
 
            chatManager.setDestAddressAndPort(s[1], Integer.parseInt(s[2]), Integer.parseInt(s[3]), Integer.parseInt(s[4]), 
                       Integer.parseInt(s[5]), Integer.parseInt(s[6])); 
 
            Object value = userTable.getValueAt(0, 0); 
            if (username.equals(value)) { 
                for (int i = 0; i < defaultTableModel.getColumnCount(); i++) 
                    s[i] = (String) defaultTableModel.getValueAt(0, i); 
 
                String msg = s[1] + ":" + s[2] + ":" + s[3] + ":" + s[4] + ":" + s[5] + ":" + s[6]; 
                chatManager.needConnect(msg); 
            } 
            else JOptionPane.showMessageDialog(null, "Error: Unable to locate address and port"); 
        } 
    } 
 
    public void dissconnectButton_actionPerformed(ActionEvent e) { 
        chatManager.closeTransmit(); 
        videoPanel.removeAll(); 
        onVideo = false; 
 
        connectButton.setEnabled(true); 
        multicastButton.setEnabled(true); 
        disconnectButton.setEnabled(false); 
        videoButton.setEnabled(false); 
        chatManager.isMulticastAddress = false; 
    } 
 
    public void videoButton_actionPerformed(ActionEvent e) { 
        if (!onVideo) { 
            String result = chatManager.startVideoTransmit(); 
            if (result != null) { 
                JOptionPane.showMessageDialog(null, "Error: " + result); 
                System.err.println("Error : " + result); 
            } 
 
            onVideo = true; 
        } else { 
            chatManager.closeVideoTransmit(); 
            videoPanel.removeAll(); 
            onVideo = false; 
        } 
    } 
 
    public void multicastButton_actionPerformed(ActionEvent e) { 
        String address = "234.55.66.88"; 
        int port = 6666; 
        int ttl = 2; 
        
        chatManager.setMulticastAndTTL(ttl); 
        chatManager.setDestAddressAndPort(address, port, port, port, port+4, port); 
        
        String result = chatManager.startAudioTransmit();
        
        if(result != null){
        	JOptionPane.showMessageDialog(null, "Error: " + result);
        }
        
        connectButton.setEnabled(false); 
        multicastButton.setEnabled(false); 
        disconnectButton.setEnabled(true); 
        videoButton.setEnabled(true);
        
    } 
    
    public int findRow(String name, String address){ 
 
        int rowCount = userTable.getRowCount(); 
        int columnCount = userTable.getColumnCount(); 
 
        for (int i = 0; i < rowCount; i++) { 
            for (int j = 0; j < columnCount; j++) { 
 
                Object value = userTable.getValueAt(i, j); 
                if (name.equals(value)) { 
 
                    for(int k = 0; k < columnCount; k++){ 
                        value = userTable.getValueAt(i, k); 
                        if (address.equals(value)){ 
                            return i; 
                        } 
                    } 
                } 
            } 
        } 
        return -1; 
    } 
 
    public String getLocalAddress(){ 
 
        InetAddress address = null; 
        try { 
            address = InetAddress.getLocalHost(); 
        } catch (UnknownHostException ex) { 
        } 
        
        String host = address.getHostAddress();
        return host; 
    } 
 
} 


